<?php

namespace Drupal\tvp\PathProcessor;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\PathProcessor\InboundPathProcessorInterface;
use Drupal\Core\PathProcessor\OutboundPathProcessorInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\path_alias\AliasRepositoryInterface;
use Drupal\views\Views;
use Symfony\Component\HttpFoundation\Request;

/**
 * Processes the inbound and outbound pager query.
 */
class TvpProcessor implements InboundPathProcessorInterface, OutboundPathProcessorInterface {


  /**
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * @var LanguageManagerInterface
   */
  protected $languageManager;


  /**
   * @var AliasRepositoryInterface
   */
  protected $aliasStorage;

  /**
   * Cache Backend.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;


  /**
   * TvpProcessor constructor.
   * @param AliasManagerInterface $alias_manager
   * @param LanguageManagerInterface $language_manager
   * @param AliasRepositoryInterface $alias_storage
   * @param CacheBackendInterface $cache_backend
   */
  public function __construct(AliasManagerInterface $alias_manager,
                              LanguageManagerInterface $language_manager,
                              AliasRepositoryInterface $alias_storage,
                              CacheBackendInterface $cache_backend) {
    $this->aliasManager = $alias_manager;
    $this->languageManager = $language_manager;
    $this->aliasStorage = $alias_storage;
    $this->cacheBackend = $cache_backend;

  }
  /**
   * {@inheritdoc}
   */
  public function processInbound($path, Request $request) {

    $path = $this->getInboundPath($path);
    return $path;
  }

  /**
   * {@inheritdoc}
   */
  public function processOutbound($path, &$options = [], Request $request = NULL, BubbleableMetadata $bubbleable_metadata = NULL) {
    $path = $this->getOutboundPath($path, $options);
    return $path;
  }

  /**
   * @param $path
   * @return string|string[]
   */
  public function getInboundPath($path) {
    if(strpos($path, "/admin") === false) {
      $lang_code = $this->languageManager->getCurrentLanguage()->getId();
      $default_lang_code = $this->languageManager->getDefaultLanguage()->getId();
      $view_path_aliases = $this->getViewPathAliases();
      if ($lang_code != $default_lang_code
        && !($this->aliasStorage->lookupByAlias($this->strReplaceFirst('/' . $lang_code,'',$path),$lang_code))
        && strpos($path,"/". $lang_code) !== false && $view_path_aliases != null) {
        foreach ($view_path_aliases as $view_path) {
          if (isset($view_path["path"])) {
            $trans_path_complete = "/" . $view_path["language"] . $view_path["trans_path"];
            if (strpos($path, $trans_path_complete) !== false) {
              $path = $this->strReplaceFirst($trans_path_complete, '/' . $lang_code . $view_path["path"], $path);
              break;
            }
          }

        }
      }
    }
    return $path;
  }

  /**
   * function to provide OutboundPath
   * @param $path
   * @param array $options
   * @return string|string[]
   */
  public function getOutboundPath($path, &$options = []) {

    $view_path_aliases = $this->getViewPathAliases();
    if(strpos($path, "/admin") === false && $path != "/" && $view_path_aliases != null ) {
      foreach ($view_path_aliases as $view_path) {
        if (isset($view_path["path"])) {
          $lang_code = isset($options['language']) ? $options['language']->getId() : NULL;
          if (strpos($path, $view_path["path"]) !== false && isset($lang_code)
              && $view_path["language"] == $lang_code) {
            $path = $this->strReplaceFirst($view_path["path"], $view_path["trans_path"], $path);
            break;
          }
        }
      }
    }
    return $path;
  }

  /**
   * @param $search
   * @param $replace
   * @param $subject
   * @return string|string[]
   */
  private function strReplaceFirst($search, $replace, $subject) {
    $pos = strpos($subject, $search);
    if ($pos !== false) {
      return substr_replace($subject, $replace, $pos, strlen($search));
    }
    return $subject;
  }

  /**
   * function to return the aliases which are for view paths
   * @return array
   */
  private function getViewPathAliases()
  {
    // Check cache.
    if ($cache = $this->cacheBackend->get('view_path_aliases_cid')) {
      $view_path_aliases = $cache->data;

    }
    else {
      $view_paths = $this->getViewPathOptions();
      $langcodes = $this->languageManager->getLanguages();
      $view_path_aliases = [];
      foreach ($view_paths as $view_path) {
        foreach ($langcodes as $langcode) {
          $url_alias = $this->aliasManager->getAliasByPath("/" . $view_path, $langcode->getId());
          $view_path_aliases[] = ['path' => "/" . $view_path, 'language' => $langcode->getId(), 'trans_path' => $url_alias];
        }
      }
      $twenty_four_hours = 60 * 60 * 24;
      // set the cache
      $this->cacheBackend->set("view_path_aliases_cid", $view_path_aliases, time() + $twenty_four_hours);
    }
    return $view_path_aliases;
  }

  /**
   * function to fetch paths of all views
   * @return array
   */
  private function getViewPathOptions()
  {
    $views_list = Views::getAllViews();
    $view_paths = [];
    foreach ($views_list as $view) {
      $displays = $view->get("display");
      foreach ($displays as $display) {
        $display_id = isset($display['id']) ? $display['id'] : null;
        if ($display_id != null && $view->getDisplay($display_id) != null && isset($view->getDisplay($display_id)['display_options'])
          && isset($view->getDisplay($display_id)['display_options']['path'])) {
          $path = $view->getDisplay($display_id)['display_options']['path'];
          if ($path != null && !empty($path && strpos($path, "admin/") === false)) {
            $view_paths[$path] = $path;
          }
        }
      }
    }
    return $view_paths;
  }
}
